#
# Copyright 2014  Infoxchange Australia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Services that can be provided to running applications.
"""

from .base import register, Service

# pylint:disable=unused-import
from .elasticsearch import Elasticsearch
from .email import Email
from .memcached import Memcache
from .postgres import PostgreSQL, PostGIS
from .proxy import Proxy
from .syslog import Syslog
from .redis import Redis
from .amqp import RabbitMQ



#
# Copyright 2014  Infoxchange Australia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Test the base Service choosing from the available providers.
"""

from forklift.base import ImproperlyConfigured
from forklift.services.base import (
    ProviderNotAvailable,
    Service,
)

from tests.base import (TestCase)


class FussyService(Service):
    """
    A service with a few providers which are only sometimes available.
    """

    def __init__(self, value, application_id):
        self.application_id = application_id
        self.value = value

    providers = (
        'one',
        'two',
    )

    # An array of values deemed available
    nice = (
        'one',
        'two',
    )

    @classmethod
    def one(cls, application_id):
        """
        One nice provider.
        """
        return cls('one', application_id)

    @classmethod
    def two(cls, application_id):
        """
        Another nice provider.
        """
        return cls('two', application_id)

    def check_available(self):
        """
        Only treat nice providers as available.
        """
        if self.value in self.nice:
            return True
        else:
            raise ProviderNotAvailable()


class TestProvide(TestCase):
    """
    Test choosing an available provider.
    """

    def test_first_available(self):
        """
        The first provider tried is available.
        """
        self.assertEqual(FussyService.provide(application_id='test').value,
                         'one')

    def test_second_available(self):
        """
        One provider isn't available, the other one is.
        """
        FussyService.nice = ('two',)
        self.assertEqual(FussyService.provide(application_id='test').value,
                         'two')

    def test_none_available(self):
        """
        No providers are available.
        """
        FussyService.nice = ()
        with self.assertRaises(ImproperlyConfigured):
            FussyService.provide(application_id='test')



#
# Copyright 2015  Infoxchange
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Redis service
"""

import logging

from .base import (
    ProviderNotAvailable,
    URLNameDescriptor,
    URLHostInfoDescriptor,
    URLService,
    port_open,
    register,
    split_host_port,
)

LOGGER = logging.getLogger(__name__)


@register('amqp')
class RabbitMQ(URLService):
    """
    A RabbitMQ/AMQP service
    """
    CONTAINER_IMAGE = 'rabbitmq'
    DEFAULT_PORT = 5672

    hosts = URLHostInfoDescriptor(default_port=DEFAULT_PORT, joiner=tuple)
    vhost = URLNameDescriptor()

    allow_override = URLService.allow_override + ('vhost',)
    allow_override_list = URLService.allow_override_list + ('hosts',)

    providers = ('localhost', 'container')

    def environment(self):
        """Environment for AMQP"""

        return {
            'AMQP_URLS': self.url_string(),
        }

    def check_available(self):
        """Check the service is available"""
        for host in self.hosts:
            if port_open(*split_host_port(host, self.DEFAULT_PORT)):
                return True

        raise ProviderNotAvailable("RabbitMQ not available: none of the hosts "
                                   "are up")

    @classmethod
    def localhost(cls, application_id):
        """RabbitMQ on the local machine"""
        return cls(urls=('amqp://localhost:{port}/{vhost}/'.format(
            port=cls.DEFAULT_PORT,
            vhost=application_id),))

    @classmethod
    def from_container(cls, application_id, container):
        """RabbitMQ as a container"""

        return cls(urls=('amqp://guest:guest@{c.host}:{c.port}//'.format(
            c=container),))
